@@ -1,5 +1,5 @@
 /* Dependency analysis
-   Copyright (C) 2000-2014 Free Software Foundation, Inc.
+   Copyright (C) 2000-2015 Free Software Foundation, Inc.
    Contributed by Paul Brook <paul@nowt.org>
 
 This file is part of GCC.
@@ -22,7 +22,7 @@ along with GCC; see the file COPYING3.  If not see
 /* There's probably quite a bit of duplication in this file.  We currently
    have different dependency checking functions for different types
    if dependencies.  Ideally these would probably be merged.  */
-   
+
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -178,14 +178,14 @@ are_identical_variables (gfc_expr *e1, gfc_expr *e2)
 
 	  /* If both are NULL, the end length compares equal, because we
 	     are looking at the same variable. This can only happen for
-	     assumed- or deferred-length character arguments.  */ 
+	     assumed- or deferred-length character arguments.  */
 
 	  if (r1->u.ss.end == NULL && r2->u.ss.end == NULL)
 	    break;
 
 	  if (gfc_dep_compare_expr (r1->u.ss.end, r2->u.ss.end) != 0)
 	    return false;
-	  
+
 	  break;
 
 	default:
@@ -206,7 +206,7 @@ gfc_dep_compare_functions (gfc_expr *e1, gfc_expr *e2, bool impure_ok)
 
   gfc_actual_arglist *args1;
   gfc_actual_arglist *args2;
-  
+
   if (e1->expr_type != EXPR_FUNCTION || e2->expr_type != EXPR_FUNCTION)
     return -2;
 
@@ -226,25 +226,25 @@ gfc_dep_compare_functions (gfc_expr *e1, gfc_expr *e2, bool impure_ok)
 	  /*  Bitwise xor, since C has no non-bitwise xor operator.  */
 	  if ((args1->expr == NULL) ^ (args2->expr == NULL))
 	    return -2;
-	  
+
 	  if (args1->expr != NULL && args2->expr != NULL
 	      && gfc_dep_compare_expr (args1->expr, args2->expr) != 0)
 	    return -2;
-	  
+
 	  args1 = args1->next;
 	  args2 = args2->next;
 	}
       return (args1 || args2) ? -2 : 0;
     }
       else
-	return -2;      
+	return -2;
 }
 
 /* Helper function to look through parens, unary plus and widening
    integer conversions.  */
 
-static gfc_expr*
-discard_nops (gfc_expr *e)
+gfc_expr *
+gfc_discard_nops (gfc_expr *e)
 {
   gfc_actual_arglist *arglist;
 
@@ -297,8 +297,8 @@ gfc_dep_compare_expr (gfc_expr *e1, gfc_expr *e2)
   if (e1 == NULL && e2 == NULL)
     return 0;
 
-  e1 = discard_nops (e1);
-  e2 = discard_nops (e2);
+  e1 = gfc_discard_nops (e1);
+  e2 = gfc_discard_nops (e2);
 
   if (e1->expr_type == EXPR_OP && e1->value.op.op == INTRINSIC_PLUS)
     {
@@ -496,7 +496,7 @@ gfc_dep_compare_expr (gfc_expr *e1, gfc_expr *e2)
 
 
 /* Return the difference between two expressions.  Integer expressions of
-   the form 
+   the form
 
    X + constant, X - constant and constant + X
 
@@ -515,8 +515,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
   if (e1->ts.type != BT_INTEGER || e2->ts.type != BT_INTEGER)
     return false;
 
-  e1 = discard_nops (e1);
-  e2 = discard_nops (e2);
+  e1 = gfc_discard_nops (e1);
+  e2 = gfc_discard_nops (e2);
 
   /* Inizialize tentatively, clear if we don't return anything.  */
   mpz_init (*result);
@@ -531,8 +531,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
   if (e1->expr_type == EXPR_OP && e1->value.op.op == INTRINSIC_PLUS)
     {
-      e1_op1 = discard_nops (e1->value.op.op1);
-      e1_op2 = discard_nops (e1->value.op.op2);
+      e1_op1 = gfc_discard_nops (e1->value.op.op1);
+      e1_op2 = gfc_discard_nops (e1->value.op.op2);
 
       /* Case 2: (X + c1) - X = c1.  */
       if (e1_op2->expr_type == EXPR_CONSTANT
@@ -542,7 +542,7 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 	  return true;
 	}
 
-      /* Case 3: (c1 + X) - X = c1. */
+      /* Case 3: (c1 + X) - X = c1.  */
       if (e1_op1->expr_type == EXPR_CONSTANT
 	  && gfc_dep_compare_expr (e1_op2, e2) == 0)
 	{
@@ -552,8 +552,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
       if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_PLUS)
 	{
-	  e2_op1 = discard_nops (e2->value.op.op1);
-	  e2_op2 = discard_nops (e2->value.op.op2);
+	  e2_op1 = gfc_discard_nops (e2->value.op.op1);
+	  e2_op2 = gfc_discard_nops (e2->value.op.op2);
 
 	  if (e1_op2->expr_type == EXPR_CONSTANT)
 	    {
@@ -597,8 +597,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
       if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_MINUS)
 	{
-	  e2_op1 = discard_nops (e2->value.op.op1);
-	  e2_op2 = discard_nops (e2->value.op.op2);
+	  e2_op1 = gfc_discard_nops (e2->value.op.op1);
+	  e2_op2 = gfc_discard_nops (e2->value.op.op2);
 
 	  if (e1_op2->expr_type == EXPR_CONSTANT)
 	    {
@@ -627,8 +627,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
   if (e1->expr_type == EXPR_OP && e1->value.op.op == INTRINSIC_MINUS)
     {
-      e1_op1 = discard_nops (e1->value.op.op1);
-      e1_op2 = discard_nops (e1->value.op.op2);
+      e1_op1 = gfc_discard_nops (e1->value.op.op1);
+      e1_op2 = gfc_discard_nops (e1->value.op.op2);
 
       if (e1_op2->expr_type == EXPR_CONSTANT)
 	{
@@ -642,8 +642,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
 	  if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_PLUS)
 	    {
-	      e2_op1 = discard_nops (e2->value.op.op1);
-	      e2_op2 = discard_nops (e2->value.op.op2);
+	      e2_op1 = gfc_discard_nops (e2->value.op.op1);
+	      e2_op2 = gfc_discard_nops (e2->value.op.op2);
 
 	      /* Case 11: (X - c1) - (X + c2) = -( c1 + c2).  */
 	      if (e2_op2->expr_type == EXPR_CONSTANT
@@ -668,8 +668,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
 	  if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_MINUS)
 	    {
-	      e2_op1 = discard_nops (e2->value.op.op1);
-	      e2_op2 = discard_nops (e2->value.op.op2);
+	      e2_op1 = gfc_discard_nops (e2->value.op.op1);
+	      e2_op2 = gfc_discard_nops (e2->value.op.op2);
 
 	      /* Case 13: (X - c1) - (X - c2) = c2 - c1.  */
 	      if (e2_op2->expr_type == EXPR_CONSTANT
@@ -685,9 +685,9 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 	{
 	  if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_MINUS)
 	    {
-	      e2_op1 = discard_nops (e2->value.op.op1);
-	      e2_op2 = discard_nops (e2->value.op.op2);
-	      
+	      e2_op1 = gfc_discard_nops (e2->value.op.op1);
+	      e2_op2 = gfc_discard_nops (e2->value.op.op2);
+
 	      /* Case 14: (c1 - X) - (c2 - X) == c1 - c2.  */
 	      if (gfc_dep_compare_expr (e1_op2, e2_op2) == 0)
 		{
@@ -702,8 +702,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
   if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_PLUS)
     {
-      e2_op1 = discard_nops (e2->value.op.op1);
-      e2_op2 = discard_nops (e2->value.op.op2);
+      e2_op1 = gfc_discard_nops (e2->value.op.op1);
+      e2_op2 = gfc_discard_nops (e2->value.op.op2);
 
       /* Case 15: X - (X + c2) = -c2.  */
       if (e2_op2->expr_type == EXPR_CONSTANT
@@ -723,8 +723,8 @@ gfc_dep_difference (gfc_expr *e1, gfc_expr *e2, mpz_t *result)
 
   if (e2->expr_type == EXPR_OP && e2->value.op.op == INTRINSIC_MINUS)
     {
-      e2_op1 = discard_nops (e2->value.op.op1);
-      e2_op2 = discard_nops (e2->value.op.op2);
+      e2_op1 = gfc_discard_nops (e2->value.op.op1);
+      e2_op2 = gfc_discard_nops (e2->value.op.op2);
 
       /* Case 17: X - (X - c2) = c2.  */
       if (e2_op2->expr_type == EXPR_CONSTANT
@@ -937,7 +937,7 @@ gfc_check_argument_var_dependency (gfc_expr *var, sym_intent intent,
   switch (expr->expr_type)
     {
     case EXPR_VARIABLE:
-      /* In case of elemental subroutines, there is no dependency 
+      /* In case of elemental subroutines, there is no dependency
          between two same-range array references.  */
       if (gfc_ref_needs_temporary_p (expr->ref)
 	  || gfc_check_dependency (var, expr, elemental == NOT_ELEMENTAL))
@@ -947,24 +947,24 @@ gfc_check_argument_var_dependency (gfc_expr *var, sym_intent intent,
 	      /* Too many false positive with pointers.  */
 	      if (!gfc_is_data_pointer (var) && !gfc_is_data_pointer (expr))
 		{
-		  /* Elemental procedures forbid unspecified intents, 
+		  /* Elemental procedures forbid unspecified intents,
 		     and we don't check dependencies for INTENT_IN args.  */
 		  gcc_assert (intent == INTENT_OUT || intent == INTENT_INOUT);
 
-		  /* We are told not to check dependencies. 
+		  /* We are told not to check dependencies.
 		     We do it, however, and issue a warning in case we find one.
-		     If a dependency is found in the case 
+		     If a dependency is found in the case
 		     elemental == ELEM_CHECK_VARIABLE, we will generate
 		     a temporary, so we don't need to bother the user.  */
-		  gfc_warning ("INTENT(%s) actual argument at %L might "
-			       "interfere with actual argument at %L.", 
-		   	       intent == INTENT_OUT ? "OUT" : "INOUT", 
+		  gfc_warning_1 ("INTENT(%s) actual argument at %L might "
+			       "interfere with actual argument at %L.",
+		   	       intent == INTENT_OUT ? "OUT" : "INOUT",
 		   	       &var->where, &expr->where);
 		}
 	      return 0;
 	    }
 	  else
-	    return 1; 
+	    return 1;
 	}
       return 0;
 
@@ -1010,17 +1010,17 @@ gfc_check_argument_var_dependency (gfc_expr *var, sym_intent intent,
 	 dependencies, as we will make a temporary anyway.  */
       if (elemental)
 	{
-	  /* If the actual arg EXPR is an expression, we need to catch 
-	     a dependency between variables in EXPR and VAR, 
+	  /* If the actual arg EXPR is an expression, we need to catch
+	     a dependency between variables in EXPR and VAR,
 	     an intent((IN)OUT) variable.  */
 	  if (expr->value.op.op1
-	      && gfc_check_argument_var_dependency (var, intent, 
-						    expr->value.op.op1, 
+	      && gfc_check_argument_var_dependency (var, intent,
+						    expr->value.op.op1,
 						    ELEM_CHECK_VARIABLE))
 	    return 1;
 	  else if (expr->value.op.op2
-		   && gfc_check_argument_var_dependency (var, intent, 
-							 expr->value.op.op2, 
+		   && gfc_check_argument_var_dependency (var, intent,
+							 expr->value.op.op2,
 							 ELEM_CHECK_VARIABLE))
 	    return 1;
 	}
@@ -1030,8 +1030,8 @@ gfc_check_argument_var_dependency (gfc_expr *var, sym_intent intent,
       return 0;
     }
 }
-  
-  
+
+
 /* Like gfc_check_argument_var_dependency, but extended to any
    array expression OTHER, not just variables.  */
 
@@ -1154,7 +1154,7 @@ gfc_are_equivalenced_arrays (gfc_expr *e1, gfc_expr *e2)
 	  /* Can these lengths be zero?  */
 	  if (fl1->length <= 0 || fl2->length <= 0)
 	    return 1;
-	  /* These can't overlap if [f11,fl1+length] is before 
+	  /* These can't overlap if [f11,fl1+length] is before
 	     [fl2,fl2+length], or [fl2,fl2+length] is before
 	     [fl1,fl1+length], otherwise they do overlap.  */
 	  if (fl1->offset + fl1->length > fl2->offset
@@ -1269,6 +1269,7 @@ gfc_check_dependency (gfc_expr *expr1, gfc_expr *expr2, bool identical)
       /* The interesting cases are when the symbols don't match.  */
       if (expr1->symtree->n.sym != expr2->symtree->n.sym)
 	{
+	  symbol_attribute attr1, attr2;
 	  gfc_typespec *ts1 = &expr1->symtree->n.sym->ts;
 	  gfc_typespec *ts2 = &expr2->symtree->n.sym->ts;
 
@@ -1284,9 +1285,13 @@ gfc_check_dependency (gfc_expr *expr1, gfc_expr *expr2, bool identical)
 		return 0;
 	    }
 
-	  /* If either variable is a pointer, assume the worst.  */
-	  /* TODO: -fassume-no-pointer-aliasing */
-	  if (gfc_is_data_pointer (expr1) || gfc_is_data_pointer (expr2))
+	  /* We have to also include target-target as ptr%comp is not a
+	     pointer but it still alias with "dt%comp" for "ptr => dt".  As
+	     subcomponents and array access to pointers retains the target
+	     attribute, that's sufficient.  */
+	  attr1 = gfc_expr_attr (expr1);
+	  attr2 = gfc_expr_attr (expr2);
+	  if ((attr1.pointer || attr1.target) && (attr2.pointer || attr2.target))
 	    {
 	      if (check_data_pointer_types (expr1, expr2)
 		    && check_data_pointer_types (expr2, expr1))
@@ -1457,7 +1462,7 @@ check_section_vs_section (gfc_array_ref *l_ar, gfc_array_ref *r_ar, int n)
     start_comparison = gfc_dep_compare_expr (l_start, r_start);
   else
     start_comparison = -2;
-      
+
   gfc_free_expr (one_expr);
 
   /* Determine LHS upper and lower bounds.  */
@@ -1550,7 +1555,7 @@ check_section_vs_section (gfc_array_ref *l_ar, gfc_array_ref *r_ar, int n)
 
 #undef IS_CONSTANT_INTEGER
 
-  /* Check for forward dependencies x:y vs. x+1:z and x:y:z vs. x:y:z+1. */
+  /* Check for forward dependencies x:y vs. x+1:z and x:y:z vs. x:y:z+1.  */
 
   if (l_dir == 1 && r_dir == 1 &&
       (start_comparison == 0 || start_comparison == -1)
@@ -1559,7 +1564,7 @@ check_section_vs_section (gfc_array_ref *l_ar, gfc_array_ref *r_ar, int n)
 
   /* Check for forward dependencies x:y:-1 vs. x-1:z:-1 and
      x:y:-1 vs. x:y:-2.  */
-  if (l_dir == -1 && r_dir == -1 && 
+  if (l_dir == -1 && r_dir == -1 &&
       (start_comparison == 0 || start_comparison == 1)
       && (stride_comparison == 0 || stride_comparison == 1))
     return GFC_DEP_FORWARD;
@@ -1583,7 +1588,7 @@ check_section_vs_section (gfc_array_ref *l_ar, gfc_array_ref *r_ar, int n)
     {
       if (l_start && IS_ARRAY_EXPLICIT (l_ar->as))
 	{
-      
+
 	  /* Check for a(high:y:-s) vs. a(z:x:-s) or
 	     a(high:y:-s vs. a(z:x:-s-1) where a has a higher bound
 	     of high, which is always at least a forward dependence.  */
@@ -2077,12 +2082,12 @@ gfc_dep_resolver (gfc_ref *lref, gfc_ref *rref, gfc_reverse *reverse)
 	  if (lref->u.c.component != rref->u.c.component)
 	    return 0;
 	  break;
-	  
+
 	case REF_SUBSTRING:
 	  /* Substring overlaps are handled by the string assignment code
 	     if there is not an underlying dependency.  */
 	  return (fin_dep == GFC_DEP_OVERLAP) ? 1 : 0;
-	
+
 	case REF_ARRAY:
 
 	  if (ref_same_as_full_array (lref, rref))
@@ -2115,7 +2120,7 @@ gfc_dep_resolver (gfc_ref *lref, gfc_ref *rref, gfc_reverse *reverse)
 	      if (lref->u.ar.dimen_type[n] == DIMEN_VECTOR
 		  || rref->u.ar.dimen_type[n] == DIMEN_VECTOR)
 		{
-		  if (lref->u.ar.dimen_type[n] == DIMEN_VECTOR 
+		  if (lref->u.ar.dimen_type[n] == DIMEN_VECTOR
 		      && rref->u.ar.dimen_type[n] == DIMEN_VECTOR
 		      && gfc_dep_compare_expr (lref->u.ar.start[n],
 					       rref->u.ar.start[n]) == 0)
@@ -2135,7 +2140,7 @@ gfc_dep_resolver (gfc_ref *lref, gfc_ref *rref, gfc_reverse *reverse)
 	      else if (rref->u.ar.dimen_type[n] == DIMEN_ELEMENT
 		       && lref->u.ar.dimen_type[n] == DIMEN_RANGE)
 		this_dep = gfc_check_element_vs_section (rref, lref, n);
-	      else 
+	      else
 		{
 		  gcc_assert (rref->u.ar.dimen_type[n] == DIMEN_ELEMENT
 			      && lref->u.ar.dimen_type[n] == DIMEN_ELEMENT);
