@@ -1,11 +1,11 @@
 #! /bin/sh
 
-# Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+# Copyright (C) 2001-2015 Free Software Foundation, Inc.
 # This file is part of GCC.
 
 # GCC is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2, or (at your option)
+# the Free Software Foundation; either version 3, or (at your option)
 # any later version.
 
 # GCC is distributed in the hope that it will be useful,
@@ -14,13 +14,12 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with GCC; see the file COPYING.  If not, write to
-# the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-# Boston MA 02110-1301, USA.
+# along with GCC; see the file COPYING3.  If not see
+# <http://www.gnu.org/licenses/>.  
 
 
 # Generate gcc's various configuration headers:
-# config.h, tconfig.h, bconfig.h, tm.h, and tm_p.h.
+# config.h, tconfig.h, bconfig.h, tm.h, libgcc_tm.h, and tm_p.h.
 # $1 is the file to generate.  DEFINES, HEADERS, and possibly
 # TARGET_CPU_DEFAULT are expected to be set in the environment.
 
@@ -41,6 +40,14 @@ header_guard=GCC_`echo ${output} | sed -e ${hg_sed_expr}`
 echo "#ifndef ${header_guard}" >> ${output}T
 echo "#define ${header_guard}" >> ${output}T
 
+# A special test to ensure that build-time files don't blindly use
+# config.h.
+if test x"$output" = x"config.h"; then
+  echo "#ifdef GENERATOR_FILE" >> ${output}T
+  echo "#error config.h is for the host, not build, machine." >> ${output}T
+  echo "#endif" >> ${output}T
+fi
+
 # Define TARGET_CPU_DEFAULT if the system wants one.
 # This substitutes for lots of *.h files.
 if [ "$TARGET_CPU_DEFAULT" != "" ]; then
@@ -56,6 +63,10 @@ done
 
 # The first entry in HEADERS may be auto-FOO.h ;
 # it wants to be included even when not -DIN_GCC.
+# Postpone including defaults.h until after the insn-*
+# headers, so that the HAVE_* flags are available
+# when defaults.h gets included.
+postpone_defaults_h="no"
 if [ -n "$HEADERS" ]; then
     set $HEADERS
     case "$1" in auto-* )
@@ -66,27 +77,41 @@ if [ -n "$HEADERS" ]; then
     if [ $# -ge 1 ]; then
 	echo '#ifdef IN_GCC' >> ${output}T
 	for file in "$@"; do
-	    echo "# include \"$file\"" >> ${output}T
+	    if test x"$file" = x"defaults.h"; then
+		postpone_defaults_h="yes"
+	    else
+		echo "# include \"$file\"" >> ${output}T
+	    fi
 	done
 	echo '#endif' >> ${output}T
     fi
 fi
 
-# If this is tm.h, now include insn-constants.h and insn-flags.h only
-# if IN_GCC is defined but neither GENERATOR_FILE nor USED_FOR_TARGET
-# is defined.  (Much of this is temporary.)
+# If this is tm.h, now include insn-flags.h only if IN_GCC is defined
+# but neither GENERATOR_FILE nor USED_FOR_TARGET is defined.  (Much of this
+# is temporary.)
 
 case $output in
     tm.h )
         cat >> ${output}T <<EOF
 #if defined IN_GCC && !defined GENERATOR_FILE && !defined USED_FOR_TARGET
-# include "insn-constants.h"
 # include "insn-flags.h"
 #endif
+#if defined IN_GCC && !defined GENERATOR_FILE
+# include "insn-modes.h"
+#endif
+#if defined IN_GCC && defined GENERATOR_FILE && !defined BITS_PER_UNIT
+#include "machmode.h"
+#endif
 EOF
     ;;
 esac
 
+# If we postponed including defaults.h, add the #include now.
+if test x"$postpone_defaults_h" = x"yes"; then
+    echo "# include \"defaults.h\"" >> ${output}T
+fi
+
 # Add multiple inclusion protection guard, part two.
 echo "#endif /* ${header_guard} */" >> ${output}T
 
