@@ -54,8 +54,7 @@ int main (void)
   return 0;
 }
 
-/* Exclude POWER8 (only POWER cpu for which vect_element_align is true)
-   because loops have vectorized before SLP gets a shot.  */
-/* { dg-final { scan-tree-dump-times "basic block vectorized" 1 "slp" { target { vect_element_align && { ! powerpc*-*-* } } } } } */
-/* { dg-final { cleanup-tree-dump "slp" } } */
+/* { dg-final { scan-tree-dump-times "basic block vectorized" 1 "slp1" { target vect_element_align } } } */
+/* { dg-final { cleanup-tree-dump "slp1" } } */
+/* { dg-final { cleanup-tree-dump "slp2" } } */
 
