@@ -1,5 +1,5 @@
 /* Tree browser.
-   Copyright (C) 2002-2013 Free Software Foundation, Inc.
+   Copyright (C) 2002-2015 Free Software Foundation, Inc.
    Contributed by Sebastian Pop <s.pop@laposte.net>
 
 This file is part of GCC.
@@ -21,8 +21,20 @@ along with GCC; see the file COPYING3.  If not see
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
+#include "hash-table.h"
+#include "hash-set.h"
+#include "machmode.h"
+#include "vec.h"
+#include "double-int.h"
+#include "input.h"
+#include "alias.h"
+#include "symtab.h"
+#include "options.h"
+#include "wide-int.h"
+#include "inchash.h"
 #include "tree.h"
 #include "tree-pretty-print.h"
+#include "print-tree.h"
 
 #define TB_OUT_FILE stdout
 #define TB_IN_FILE stdin
@@ -45,7 +57,7 @@ struct tb_command {
   TB_CODE comm_code;
 };
 
-#define DEFTBCODE(code, str, help) { help, str, sizeof(str) - 1, code },
+#define DEFTBCODE(code, str, help) { help, str, sizeof (str) - 1, code },
 static const struct tb_command tb_commands[] =
 {
 #include "tree-browser.def"
@@ -94,14 +106,36 @@ static tree TB_next_expr (tree);
 static tree TB_up_expr (tree);
 static tree TB_first_in_bind (tree);
 static tree TB_last_in_bind (tree);
-static int  TB_parent_eq (const void *, const void *);
 static tree TB_history_prev (void);
 
 /* FIXME: To be declared in a .h file.  */
 void browse_tree (tree);
 
+/* Hashtable helpers.  */
+struct tree_upper_hasher : pointer_hash<tree_node>
+{
+  static inline bool equal (const value_type &, const compare_type &);
+};
+
+inline bool
+tree_upper_hasher::equal (const value_type &parent, const compare_type &node)
+{
+  if (parent == NULL || node == NULL)
+    return 0;
+
+  if (EXPR_P (parent))
+    {
+      int n = TREE_OPERAND_LENGTH (parent);
+      int i;
+      for (i = 0; i < n; i++)
+	if (node == TREE_OPERAND (parent, i))
+	  return true;
+    }
+  return false;
+}
+
 /* Static variables.  */
-static htab_t TB_up_ht;
+static hash_table<tree_upper_hasher> *TB_up_ht;
 static vec<tree, va_gc> *TB_history_stack;
 static int TB_verbose = 1;
 
@@ -134,7 +168,7 @@ browse_tree (tree begin)
 
   /* Store in a hashtable information about previous and upper statements.  */
   {
-    TB_up_ht = htab_create (1023, htab_hash_pointer, &TB_parent_eq, NULL);
+    TB_up_ht = new hash_table<tree_upper_hasher> (1023);
     TB_update_up (head);
   }
 
@@ -612,7 +646,8 @@ browse_tree (tree begin)
     }
 
  ret:;
-  htab_delete (TB_up_ht);
+  delete TB_up_ht;
+  TB_up_ht = NULL;
   return;
 }
 
@@ -658,7 +693,7 @@ TB_up_expr (tree node)
   if (node == NULL_TREE)
     return NULL_TREE;
 
-  res = (tree) htab_find (TB_up_ht, node);
+  res = TB_up_ht->find (node);
   return res;
 }
 
@@ -724,7 +759,7 @@ store_child_info (tree *tp, int *walk_subtrees ATTRIBUTE_UNUSED,
 		  void *data ATTRIBUTE_UNUSED)
 {
   tree node;
-  void **slot;
+  tree_node **slot;
 
   node = *tp;
 
@@ -736,8 +771,8 @@ store_child_info (tree *tp, int *walk_subtrees ATTRIBUTE_UNUSED,
       for (i = 0; i < n; i++)
 	{
 	  tree op = TREE_OPERAND (node, i);
-	  slot = htab_find_slot (TB_up_ht, op, INSERT);
-	  *slot = (void *) node;
+	  slot = TB_up_ht->find_slot (op, INSERT);
+	  *slot = node;
 	}
     }
 
@@ -745,28 +780,6 @@ store_child_info (tree *tp, int *walk_subtrees ATTRIBUTE_UNUSED,
   return NULL_TREE;
 }
 
-/* Function used in TB_up_ht.  */
-
-static int
-TB_parent_eq (const void *p1, const void *p2)
-{
-  const_tree const node = (const_tree)p2;
-  const_tree const parent = (const_tree) p1;
-
-  if (p1 == NULL || p2 == NULL)
-    return 0;
-
-  if (EXPR_P (parent))
-    {
-      int n = TREE_OPERAND_LENGTH (parent);
-      int i;
-      for (i = 0; i < n; i++)
-	if (node == TREE_OPERAND (parent, i))
-	  return 1;
-    }
-  return 0;
-}
-
 /* Update information about upper expressions in the hash table.  */
 
 static void
