@@ -1,5 +1,6 @@
 ! PR fortran/49792
 ! { dg-do run }
+! { dg-options "-std=f2003 -fall-intrinsics" }
 
 subroutine reverse(n, a)
   integer :: n
@@ -10,9 +11,12 @@ subroutine reverse(n, a)
 end subroutine reverse
 
 program pr49792
-  real(kind=8) :: a(16) = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]
-  real(kind=8) :: b(16)
-  b(:) = a(16:1:-1)
-  call reverse (16,a)
-  if (any (a.ne.b)) call abort
+  integer :: b(16)
+  integer, allocatable :: a(:)
+  b = 1
+!$omp parallel workshare
+  a = b
+!$omp end parallel workshare
+  if (size(a).ne.size(b)) call abort()
+  if (any (a.ne.b)) call abort()
 end program pr49792
