@@ -1,5 +1,5 @@
 /* System.java -- useful methods to interface with the system
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2012
    Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
@@ -45,6 +45,7 @@ import gnu.classpath.VMStackWalker;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintStream;
+import java.io.Console;
 import java.nio.channels.Channel;
 import java.nio.channels.spi.SelectorProvider;
 import java.util.AbstractCollection;
@@ -96,6 +97,8 @@ public final class System
    */
   public static final PrintStream out = VMSystem.makeStandardOutputStream();
 
+  private static final String LINE_SEPARATOR = SystemProperties.getProperty("line.separator");
+
   /**
    * The standard output PrintStream.  This is assigned at startup and
    * starts its life perfectly valid. Although it is marked final, you can
@@ -134,7 +137,7 @@ public final class System
     SecurityManager sm = SecurityManager.current; // Be thread-safe.
     if (sm != null)
       sm.checkPermission(new RuntimePermission("setIO"));
-    
+
     VMSystem.setIn(in);
   }
 
@@ -151,7 +154,7 @@ public final class System
   {
     SecurityManager sm = SecurityManager.current; // Be thread-safe.
     if (sm != null)
-      sm.checkPermission(new RuntimePermission("setIO"));    
+      sm.checkPermission(new RuntimePermission("setIO"));
     VMSystem.setOut(out);
   }
 
@@ -202,15 +205,15 @@ public final class System
     // preload it here.
     if (SecurityManager.current == null)
       {
-	try
-	  {
-	    Class.forName("java.security.Security");
-	  }
-	catch (ClassNotFoundException e)
-	  {
-	  }
+        try
+          {
+            Class.forName("java.security.Security");
+          }
+        catch (ClassNotFoundException e)
+          {
+          }
       }
-    
+
     SecurityManager.current = sm;
   }
 
@@ -237,8 +240,8 @@ public final class System
   {
     return VMSystem.currentTimeMillis();
   }
-  
-  /** 
+
+  /**
    * <p>
    * Returns the current value of a nanosecond-precise system timer.
    * The value of the timer is an offset relative to some arbitrary fixed
@@ -261,7 +264,7 @@ public final class System
    * </p>
    *
    * @return the time of a system timer in nanoseconds.
-   * @since 1.5 
+   * @since 1.5
    */
   public static long nanoTime()
   {
@@ -365,7 +368,7 @@ public final class System
    * <dt>gnu.java.io.encoding_scheme_alias.iso-latin-_?</dt> <dd>8859_?</dd>
    * <dt>gnu.java.io.encoding_scheme_alias.latin?</dt>       <dd>8859_?</dd>
    * <dt>gnu.java.io.encoding_scheme_alias.utf-8</dt>        <dd>UTF8</dd>
-   * <dt>gnu.java.util.zoneinfo.dir</dt>	<dd>Root of zoneinfo tree</dd>
+   * <dt>gnu.java.util.zoneinfo.dir</dt>        <dd>Root of zoneinfo tree</dd>
    * <dt>gnu.javax.print.server</dt>     <dd>Hostname of external CUPS server.</dd>
    * </dl>
    *
@@ -549,7 +552,7 @@ public final class System
 
     if (environmentMap == null)
       {
-	Map<String,String> variables = new EnvironmentMap();
+        Map<String,String> variables = new EnvironmentMap();
         List<String> environ = (List<String>)VMSystem.environ();
         for (String envEntry : environ)
           {
@@ -558,16 +561,16 @@ public final class System
               {
                 // it's perfectly legal that some entries may be in the form
                 // key=value=value=value
-                int equalSignIndex = envEntry.indexOf('=');            
+                int equalSignIndex = envEntry.indexOf('=');
                 String key = envEntry.substring(0, equalSignIndex);
                 String value = envEntry.substring(equalSignIndex + 1);
                 variables.put(key, value);
               }
           }
-        
+
         environmentMap = Collections.unmodifiableMap(variables);
       }
-    
+
     return environmentMap;
   }
 
@@ -703,6 +706,24 @@ public final class System
     return SelectorProvider.provider().inheritedChannel();
   }
 
+  /*
+   * @since 1.6
+   */
+  public static Console console()
+  {
+    return Console.console();
+  }
+
+  /**
+   * Returns the system-dependent line separator.
+   *
+   * @return the system-dependent line separator.
+   */
+  public static String lineSeparator()
+  {
+    return LINE_SEPARATOR;
+  }
+
   /**
    * This is a specialised <code>Collection</code>, providing
    * the necessary provisions for the collections used by the
@@ -731,7 +752,7 @@ public final class System
     {
       c = coll;
     }
-        
+
     /**
      * Blocks queries containing a null object or an object which
      * isn't of type <code>String</code>.  All other queries
@@ -745,15 +766,15 @@ public final class System
     public boolean contains(Object obj)
     {
       if (obj == null)
-	  throw new
-	    NullPointerException("This collection does not support " +
-				 "null values.");
+          throw new
+            NullPointerException("This collection does not support " +
+                                 "null values.");
       if (!(obj instanceof String))
-	  throw new
-	    ClassCastException("This collection only supports Strings.");
+          throw new
+            ClassCastException("This collection only supports Strings.");
       return c.contains(obj);
     }
-    
+
     /**
      * Blocks queries where the collection contains a null object or
      * an object which isn't of type <code>String</code>.  All other
@@ -768,15 +789,15 @@ public final class System
     public boolean containsAll(Collection<?> coll)
     {
       for (Object o: coll)
-	{
-	  if (o == null)
-	      throw new
-		NullPointerException("This collection does not support " +
-				     "null values.");
-	  if (!(o instanceof String))
-	      throw new
-		ClassCastException("This collection only supports Strings.");
-	}
+        {
+          if (o == null)
+              throw new
+                NullPointerException("This collection does not support " +
+                                     "null values.");
+          if (!(o instanceof String))
+              throw new
+                ClassCastException("This collection only supports Strings.");
+        }
       return c.containsAll(coll);
     }
 
@@ -790,7 +811,7 @@ public final class System
     {
       return c.iterator();
     }
-    
+
     /**
      * Blocks the removal of elements from the collection.
      *
@@ -802,15 +823,15 @@ public final class System
     public boolean remove(Object key)
     {
       if (key == null)
-	  throw new
-	    NullPointerException("This collection does not support " +
-				 "null values.");
+          throw new
+            NullPointerException("This collection does not support " +
+                                 "null values.");
       if (!(key instanceof String))
-	  throw new
-	    ClassCastException("This collection only supports Strings.");
+          throw new
+            ClassCastException("This collection only supports Strings.");
       return c.contains(key);
-    }	
-        
+    }
+
     /**
      * Blocks the removal of all elements in the specified
      * collection from the collection.
@@ -824,18 +845,18 @@ public final class System
     public boolean removeAll(Collection<?> coll)
     {
       for (Object o: coll)
-	{
-	  if (o == null)
-	      throw new
-		NullPointerException("This collection does not support " +
-				     "null values.");
-	  if (!(o instanceof String))
-	    throw new
-	      ClassCastException("This collection only supports Strings.");
-	}
+        {
+          if (o == null)
+              throw new
+                NullPointerException("This collection does not support " +
+                                     "null values.");
+          if (!(o instanceof String))
+            throw new
+              ClassCastException("This collection only supports Strings.");
+        }
       return c.removeAll(coll);
     }
-    
+
     /**
      * Blocks the retention of all elements in the specified
      * collection from the collection.
@@ -849,15 +870,15 @@ public final class System
     public boolean retainAll(Collection<?> coll)
     {
       for (Object o: coll)
-	{
-	  if (o == null)
-	      throw new
-		NullPointerException("This collection does not support " +
-				     "null values.");
-	  if (!(o instanceof String))
-	    throw new
-	      ClassCastException("This collection only supports Strings.");
-	}
+        {
+          if (o == null)
+              throw new
+                NullPointerException("This collection does not support " +
+                                     "null values.");
+          if (!(o instanceof String))
+            throw new
+              ClassCastException("This collection only supports Strings.");
+        }
       return c.containsAll(coll);
     }
 
@@ -877,14 +898,14 @@ public final class System
   /**
    * This is a specialised <code>HashMap</code>, which
    * prevents the addition or querying of anything other than
-   * <code>String</code> objects. 
+   * <code>String</code> objects.
    *
    * @author Andrew John Hughes (gnu_andrew@member.fsf.org)
    */
   static class EnvironmentMap
     extends HashMap<String,String>
   {
-    
+
     /**
      * Cache the entry set.
      */
@@ -915,7 +936,7 @@ public final class System
      * @param m the map to be added to this.
      * @throws NullPointerException if a key or value is null.
      * @throws ClassCastException if a key or value is not a String.
-     */    
+     */
     EnvironmentMap(Map<String,String> m)
     {
       super(m);
@@ -933,14 +954,14 @@ public final class System
     public boolean containsKey(Object key)
     {
       if (key == null)
-	throw new
-	  NullPointerException("This map does not support null keys.");
+        throw new
+          NullPointerException("This map does not support null keys.");
       if (!(key instanceof String))
-	throw new
-	  ClassCastException("This map only allows queries using Strings.");
+        throw new
+          ClassCastException("This map only allows queries using Strings.");
       return super.containsKey(key);
     }
-    
+
     /**
      * Blocks queries using a null or non-<code>String</code> value.
      * All other queries are forwarded to the superclass.
@@ -952,11 +973,11 @@ public final class System
     public boolean containsValue(Object value)
     {
       if (value == null)
-	  throw new
-	    NullPointerException("This map does not support null values.");
+          throw new
+            NullPointerException("This map does not support null values.");
       if (!(value instanceof String))
-	throw new
-	  ClassCastException("This map only allows queries using Strings.");
+        throw new
+          ClassCastException("This map only allows queries using Strings.");
       return super.containsValue(value);
     }
 
@@ -985,14 +1006,14 @@ public final class System
     public String get(Object key)
     {
       if (key == null)
-	throw new
-	  NullPointerException("This map does not support null keys.");
+        throw new
+          NullPointerException("This map does not support null keys.");
       if (!(key instanceof String))
-	throw new
-	  ClassCastException("This map only allows queries using Strings.");
+        throw new
+          ClassCastException("This map only allows queries using Strings.");
       return super.get(key);
     }
-    
+
     /**
      * Returns a set view of the keys, with the same
      * provisions as for the underlying map.
@@ -1021,13 +1042,13 @@ public final class System
     public String put(String key, String value)
     {
       if (key == null)
-	throw new NullPointerException("A new key is null.");
+        throw new NullPointerException("A new key is null.");
       if (value == null)
-	throw new NullPointerException("A new value is null.");
+        throw new NullPointerException("A new value is null.");
       if (!(key instanceof String))
-	throw new ClassCastException("A new key is not a String.");
+        throw new ClassCastException("A new key is not a String.");
       if (!(value instanceof String))
-	throw new ClassCastException("A new value is not a String.");
+        throw new ClassCastException("A new value is not a String.");
       return super.put(key, value);
     }
 
@@ -1043,14 +1064,14 @@ public final class System
     public String remove(Object key)
     {
       if (key == null)
-	throw new
-	  NullPointerException("This map does not support null keys.");
+        throw new
+          NullPointerException("This map does not support null keys.");
       if (!(key instanceof String))
-	throw new
-	  ClassCastException("This map only allows queries using Strings.");
+        throw new
+          ClassCastException("This map only allows queries using Strings.");
       return super.remove(key);
     }
-    
+
     /**
      * Returns a collection view of the values, with the same
      * provisions as for the underlying map.
@@ -1063,7 +1084,7 @@ public final class System
         values = new EnvironmentCollection(super.values());
       return values;
     }
-    
+
   }
 
   /**
