@@ -1,11 +1,14 @@
+// { dg-require-namedlocale "en_HK" }
+// { dg-require-namedlocale "de_DE" }
+
 // 2001-09-21 Benjamin Kosnik  <bkoz@redhat.com>
 
-// Copyright (C) 2001, 2002, 2003 Free Software Foundation
+// Copyright (C) 2001-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,9 +17,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 22.2.5.1.1 time_get members
 
@@ -33,8 +35,8 @@ void test02()
 
   // basic construction and sanity checks.
   locale loc_c = locale::classic();
-  locale loc_hk = __gnu_test::try_named_locale("en_HK");
-  locale loc_de = __gnu_test::try_named_locale("de_DE");
+  locale loc_hk = locale("en_HK");
+  locale loc_de = locale("de_DE");
   VERIFY( loc_hk != loc_c );
   VERIFY( loc_hk != loc_de );
 
@@ -50,7 +52,7 @@ void test02()
   ios_base::iostate errorstate = good;
 
   // create "C" time objects
-  const tm time_bday = { 0, 0, 12, 4, 3, 71, 0, 93, 0 };
+  const tm time_bday = __gnu_test::test_tm(0, 0, 12, 4, 3, 71, 0, 93, 0);
 
   // inspection of named locales, de_DE
   iss.imbue(loc_de);
