@@ -1,6 +1,6 @@
-// { dg-options "-std=gnu++0x" }
+// { dg-options "-std=gnu++11" }
 
-// Copyright (C) 2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2005-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -17,16 +17,12 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-
 #include <deque>
 #include <testsuite_hooks.h>
 #include <testsuite_rvalref.h>
 
 using namespace __gnu_test;
 
-// According to n1771, there should be two resizes, with and without
-// parameter. We only have one at present, whose second parameter defaults
-// to a default-constructed object.
 void
 test01()
 {
@@ -38,30 +34,17 @@ test01()
   a.resize(98);
   a.resize(99);
   a.resize(100);
-#ifndef _GLIBCXX_DEBUG
-  VERIFY( copycounter::copycount == 100 );
-#else
-  VERIFY( copycounter::copycount == 100 + 4 );
-#endif
+  VERIFY( copycounter::copycount == 0 );
+
   a.resize(99);
   a.resize(0);
-#ifndef _GLIBCXX_DEBUG
-  VERIFY( copycounter::copycount == 100 );
-#else
-  VERIFY( copycounter::copycount == 100 + 6 );
-#endif
+  VERIFY( copycounter::copycount == 0 );
+
   a.resize(100);
-#ifndef _GLIBCXX_DEBUG
-  VERIFY( copycounter::copycount == 200 );
-#else
-  VERIFY( copycounter::copycount == 200 + 7 );
-#endif
+  VERIFY( copycounter::copycount == 0 );
+
   a.clear();
-#ifndef _GLIBCXX_DEBUG
-  VERIFY( copycounter::copycount == 200 );
-#else
-  VERIFY( copycounter::copycount == 200 + 7 );
-#endif
+  VERIFY( copycounter::copycount == 0 );
 }
 
 
