@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2013 Free Software Foundation, Inc.
+// Copyright (C) 2003-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -63,11 +63,12 @@ void test01()
   locale loc(locale::classic(), cvt);
 
   filebuf fb;
+  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
   fb.sputc('a');
-  fb.pubsync(); // Does not call unshift()
+  fb.in_avail(); // showmanyc() should have no effect on close().
 
   VERIFY( !cvt->unshift_called );
   fb.close();
