@@ -1,6 +1,6 @@
 // 2003-02-26 Benjamin Kosnik <bkoz@redhat.com>
 
-// Copyright (C) 2003-2013 Free Software Foundation, Inc.
+// Copyright (C) 2003-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,8 +25,15 @@
 int main()
 {
   using namespace __gnu_test;
-  // cplus-dem FAIL
-  verify_demangle("_Z1fA37_iPS_", "f(int [37], int (*) [37])");
+
+ // cplus-dem FAIL
+  // Equivalent as considered order-insensitive?
+  // standard text, icc FAIL
+  //  verify_demangle("_ZlsRKU3fooU4bart1XS0_", 
+  //     "operator<<(X const foo bart&, X const foo bart)");
+  // new __cxa_demangle
+  verify_demangle("_ZlsRKU3fooU4bart1XS0_", 
+		  "operator<<(X bart foo const&, X bart)");
 
   return 0;
 }
